/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent.jcmd.backend.internal.model;

import com.redhat.thermostat.agent.utils.management.MXBeanConnection;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class JCMDBindingTest {

    private MXBeanConnection connection;
    private MBeanServerConnection serverConnection;

    @Before
    public void setup() {
        connection = mock(MXBeanConnection.class);
        serverConnection = mock(MBeanServerConnection.class);
        when(connection.get()).thenReturn(serverConnection);

    }

    @Test
    public void testJCMDParsesJVMFlags() throws Exception {

        when(serverConnection.invoke(eq(new ObjectName("com.sun.management:type=DiagnosticCommand")),
                                     eq("vmFlags"), any(String[][].class),
                                     any(String[].class))).
                thenReturn("+UnlockDiagnosticVMOptions -something");

        JCMDBinding binding = new JCMDBinding(connection);

        assertTrue(binding.canCollectDiagnosticData());
    }

    @Test
    public void testJCMDRetrievesAllGCClassStats() throws Exception {

        ArgumentCaptor<String[][]> captor = ArgumentCaptor.forClass(String[][].class);

        when(serverConnection.invoke(eq(new ObjectName("com.sun.management:type=DiagnosticCommand")),
                                     eq("gcClassStats"), captor.capture(),
                                     any(String[].class))).thenReturn("42");

        JCMDBinding binding = new JCMDBinding(connection);
        assertEquals("42", binding.getGCStats());

        String parameters = captor.getValue()[0][0];
        assertEquals("InstBytes,InstCount,KlassBytes,CpAll,annotations," +
                     "MethodCount,MethodAll,ROAll,RWAll,ClassName,Bytecodes," +
                     "ClassLoader,Total", parameters);
    }
}
