/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.byteman.agent.internal;

import java.net.URI;
import java.util.Arrays;
import java.util.logging.Logger;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.common.plugin.BackendConfigurationUtil;
import com.redhat.thermostat.common.plugin.PluginDAOBase;
import com.redhat.thermostat.common.plugin.SystemID;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.lang.schema.JSONService;

import com.redhat.thermostat.vm.byteman.agent.VmBytemanStatus;

class VmBytemanStatusStore extends PluginDAOBase<VmBytemanStatus> {

    private static final String STATUS_PATH = "status";
    private static final Logger LOGGER = LoggingUtils.getLogger(VmBytemanStatusStore.class);
    private final HttpRequestService httpRequestService;
    private final BackendConfigurationUtil pluginConfig;
    private final SystemID systemId;
    private final JSONService jsonService;

    VmBytemanStatusStore(HttpRequestService httpRequestService,
                         BackendConfigurationUtil pluginConfig,
                         SystemID systemId,
                         JSONService jsonService) {
        this.httpRequestService = httpRequestService;
        this.pluginConfig = pluginConfig;
        this.systemId = systemId;
        this.jsonService = jsonService;
    }

    @Override
    protected HttpRequestService getHttpRequestService() {
        return httpRequestService;
    }

    @Override
    protected BackendConfigurationUtil getConfig() {
        return pluginConfig;
    }

    @Override
    protected URI getPostURI(URI basepath, VmBytemanStatus obj) {
        return basepath.resolve(STATUS_PATH + "/systems/" + systemId.getSystemID() + "/jvms/" + obj.getJvmId());
    }

    @Override
    protected String serialise(VmBytemanStatus obj) {
        return jsonService.serialiase(Arrays.asList(obj));
    }

    @Override
    protected Logger getLogger() {
        return LOGGER;
    }

}
