/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.memory.agent.internal.models;

import java.net.URI;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.common.plugin.BackendConfigurationUtil;
import com.redhat.thermostat.common.plugin.PluginDAOBase;
import com.redhat.thermostat.common.plugin.SystemID;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.lang.schema.JSONService;
import com.redhat.thermostat.vm.memory.agent.model.VmTlabStat;

@Component
@Service(value = VmTlabStatDAO.class)
public class VmTlabStatDAOImpl extends PluginDAOBase<VmTlabStat> implements VmTlabStatDAO {

    private static final Logger logger = LoggingUtils.getLogger(VmTlabStatDAOImpl.class);
    private static final String PLUGIN_ID = "vm-memory";

    private final ConfigurationCreator configurationCreator;
    private BackendConfigurationUtil config;

    @Reference
    private ConfigurationInfoSource configurationInfoSource;
    
    @Reference
    private HttpRequestService httpRequestService;

    @Reference
    private JSONService jsonService;

    @Reference
    private SystemID systemID;

    public VmTlabStatDAOImpl() throws Exception {
        this(new ConfigurationCreator());
    }

    VmTlabStatDAOImpl(ConfigurationCreator configurationCreator) throws Exception {
        this.configurationCreator = configurationCreator;
    }

    @Activate
    void activate() throws Exception {
        config = configurationCreator.create(configurationInfoSource);
    }

    @Override
    public void put(final VmTlabStat stat) {
//      TODO: Re-enable when web-gateway service for TLAB stats is available
//      Also see VmTlabStatDAOTest for disabled tests
        return;
//        try {
//            String json = jsonService.toJson(Arrays.asList(stat));
//            httpRequestService.sendHttpRequest(json, gatewayURL, Method.POST);
//        } catch (RequestFailedException | IOException e) {
//            logger.log(Level.WARNING, "Failed to send VmTlabStat to Web Gateway", e);
//        }
    }

    @Override
    protected URI getPostURI(final URI basepath, final VmTlabStat obj) {
        return basepath.resolve("systems/" + systemID.getSystemID() + "/jvms/" + obj.getJvmId());
    }

    @Override
    protected String serialise(VmTlabStat obj) {
        return jsonService.serialiase(Arrays.asList(obj));
    }

    @Override
    protected HttpRequestService getHttpRequestService() {
        return httpRequestService;
    }

    @Override
    protected BackendConfigurationUtil getConfig() {
        return config;
    }

    protected void bindConfigurationInfoSource(ConfigurationInfoSource configurationInfoSource) {
        this.configurationInfoSource = configurationInfoSource;
    }

    protected void bindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = httpRequestService;
    }

    protected void unbindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = null;
        logger.log(Level.INFO, "Unbound HTTP service. Further attempts to store data will fail until bound again.");
    }

    protected void bindJsonService(JSONService jsonService) {
        this.jsonService = jsonService;
    }

    void bindSystemID(final SystemID id) {
        this.systemID = id;
    }

    protected Logger getLogger() {
        return logger;
    }

    // For testing purposes
    static class ConfigurationCreator {

        BackendConfigurationUtil create(ConfigurationInfoSource source) {
            return new BackendConfigurationUtil(source, PLUGIN_ID);
        }
    }
}

